;
; dir.asm - directory handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

XIT_dir_first	.proc
		lsr
		sta direntry_first_exp
                .if SUPERCPU=1
                ror
                sta direntry_raw_exp
                .else
                ror direntry_raw_exp
                .fi
		lda path_exp+1		;"$0" hack...
		eor #"0"
		ora path_exp+2
		bne dollar0
		sta path_exp+1
dollar0
		#lda0
                sta direntry_longmode_exp

kojq		jsr OP_short_path
		cmp #"="
		bne nospecial
		jsr OP_short_path
		eor #"P"
		beq qx
                cmp #"P"^"T"
                bne kojq
		sta direntry_longmode_exp
                geq kojq
qx              sta direntry_partnum
		jsr OP_short_path
		jsr OP_get_current_drive_type
                pha
		and #FS_MASK
		eor #FS_CFS
		cmp #1
		jsr LWOP_reporterror_71_xit;-=> partition listing only for CFS!

		lda #DIRMODE_PARTITION_START
		sta direntry_mode_exp
		ldy #0
		sty next_sector
		sty next_sector+1
		sty next_sector+2
                pla
		bmi *+3			;LBA?
                iny
		sty next_sector+3
		lda #255
		gne cke
nospecial
		lda #RESOLV_LAST_REMAINS
		jsr OP_XIT_resolv_path_part;-=> error
		lda path_partition_exp
		bne cke
		#ldatax current_drive_exp
		lda drive_workpartition_exp,x
		clc
		adc #1
cke		sta direntry_partition_exp
		#lda0
		sta direntry_used_exp
		sta direntry_used_exp+1

		ldy #"="
		jsr OP_path_searchtype
		bcc sf
		lda #"*"
		sta path_searchtype_exp
		#lda0
		sta path_searchtype_exp+1
sf
		lda path_exp		;Empty="*"
		bne tko
		sta path_exp+1
		lda #"*"
		sta path_exp
tko					;set up wildcard optimization
		ldx #0
		lda path_exp
		eor #"*"
		ora path_exp+1
		bne *+4
		ldx #128		;no name wildcard
		lda path_searchtype_exp
		eor #"*"
		ora path_searchtype_exp+1
		bne *+3
		inx			;no type wildcard
		#stx_l_a direntry_wildcard_exp
		rts
		.pend
;------------
XIT_dirlp2	.proc

		.if SUPERCPU=1
		tsc
                .else
		tsx
		txa
                .fi
                sec
		sbc #31+2
		sta direntry_spc_exp

		jsr XIT_upload_next_sector;-=> error
		ldy #1			;1 sector
		jsr OP_read
		jsr LWOP_reporterror_read_xit;-=> error

		lda #15			;16 directory entrys
		sta direntry_number_exp
dirlp		jsr dirnotlast
		txa
                beq jw
                clc
		adc start_zeropage+5
		sta start_zeropage+5
		bcc jw
		inc start_zeropage+6
jw
		#dec_l_a direntry_number_exp
		bpl dirlp

		jsr ata_pi_nomore_data
		jsr LWOP_reporterror_read_xit;-=> error

		jsr next_sector0
		bne bzb

		jsr summary
		txa
		clc
		adc start_zeropage+5
		sta start_zeropage+5
		bcc bzb
		inc start_zeropage+6
bzb     	rts
;-------------------------
summary 	ldx #255
		#stx_l_a direntry_first_exp	;end of directory
                inx

		lda direntry_raw_exp
                bmi bzb

		lda direntry_mode_exp
                bmi bzb
		eor #DIRMODE_PARTITION_END
		cmp #1
                bcs ce

                lda setup_atribut2_exp
                asl
                bcc ce
                #lda0
                pha
                pha
                ldy #1
                ldx #2
		jmp start_zeropage
ce

		.if SUPERCPU=1
                pea $0101
                .else
		lda #1
		pha			;1
		pha			;2
                .fi
ce2
		bcs se
                lda direntry_partnum
                pha
                #lda0
                gcc se2

se		lda direntry_used_exp
		pha			;3
		lda direntry_used_exp+1
se2		pha			;4
  		ldx #12
dfsvcxc		lda blkused_text,x	;17
		bcs vd
		lda prtused_text,x
vd		pha
		dex
		bpl dfsvcxc
		ldx #12
gv		pha			;29
		dex
		bne gv
                .if SUPERCPU=1
                phx
                phx
                phx
                .else
		txa
		pha			;30
		pha			;31
		pha			;32
                .fi
		ldy #31
                ldx #32
		jmp start_zeropage
;----------------------------
dirnotlast	jsr OP_read_to_direntry

		lda direntry_first_exp
		beq notdirlabel

;		lda direntry_first_exp
		ldy #29
		lsr
		beq hnuhi		;load, skip $0401
					;lda #1
		pha			;1
		lda #4
		pha			;2
		ldy #31

hnuhi		#ldatax direntry_attrib_exp
		and #FTYPE_MASK | FATTR_CLOSED
		eor #FTYPE_DIR
                beq tip
		jsr flushdir
		sec
		gcs LWOP_reporterror_71_xit;-=> directory error

tip             sta direntry_first_exp

		txa
		and #FATTR_READ
                bne jo
                cpy #31
                bne ke
                pla
                pla
ke              jsr flushdir
		sec 
                gcs LWOP_reporterror_27_xit;-=> no access
jo

                lda direntry_raw_exp
		bmi raw

		.if SUPERCPU=1
                pea $0101
                .else
		lda #1
		pha			;3
		pha			;4
                .fi

		lda direntry_partition_exp
		pha			;5 currentpartition number
		#lda0
		pha			;6
		#psr $1223		;7,8 rvson"
		ldx #240

dcb		lda direntry_name_exp-240,x
		pha			;24
		inx
		bne dcb
		ldx #7
dfsvcx		lda dirlabel_text,x
		pha			;32
		dex
		bpl dfsvcx
                .if SUPERCPU=1
                tyx
                .else
		tya
		tax
                .fi
		inx
		jmp start_zeropage

raw		pla
		pla
                lda #"I"
                pha
                ldy #32
                .byte SKIP2
raw2		ldy #31
		ldx #0
lu      	lda direntry_exp,x
                pha
                inx
                cpx #32
                bcc lu
                .if SUPERCPU=1
                tyx
                .else
                tya
                tax
                .fi
                inx
		jmp start_zeropage
;----------------------------
nomatch2	#dec_l_a direntry_partnum
nomatch		ldx #0
		rts
;----------------------------
notdirlabel	lda direntry_raw_exp
		bmi raw2
		lda direntry_hidden_exp
		bmi nomatch		;hidden entry...
		lda direntry_attrib_exp
		and #FATTR_CLOSED | FTYPE_MASK
		beq nomatch		;free entry...
		and #FTYPE_MASK
		eor #FTYPE_RELATIVE
		bne uie			;to report correct filesize
		sta direntry_recordsize_exp;for relative files...
uie		lda direntry_wildcard_exp
		bmi dirskip1

		jsr check_wildcards
		bcs nomatch2

		lda direntry_wildcard_exp
dirskip1	lsr
		bcs dirskip2

		inc current_path+1
		jsr check_wildtype
		dec current_path+1
		bcs nomatch2
		sec

dirskip2	.if SUPERCPU=1
                pea $0101
                .else
		lda #1
		pha			;1
		pha			;2
                .fi

		.if DIRECTORY_BLOCKSIZE_256=1
		lda direntry_size_exp
		adc #<(255-1)
		lda direntry_size_exp+1
		adc #>(255-1)
		tay
		lda direntry_size_exp+2
		adc #0
		tax
		lda direntry_size_exp+3
		adc #0
		beq *+6
		ldx #255
		ldy #255
		tya
                pha                     ;3
		.else
		lda direntry_size_exp
		adc #<(511-1)
		lda direntry_size_exp+1
		adc #>(511-1)
		tay
		lda direntry_size_exp+2
		adc #0
		tax
		lda direntry_size_exp+3
		adc #0
		lsr a
		beq *+6
		ldx #255
		ldy #255
		txa
		ror a
		tax
		tya
		ror a
		pha			;3
		tay
		.fi
		clc
		adc direntry_used_exp
		sta direntry_used_exp
		txa
		adc direntry_used_exp+1
		bcc noovf		;In case of overflow 65535
		lda #255
		sta direntry_used_exp
noovf		sta direntry_used_exp+1

		txa
		pha			;4
		cpy #<10
		sbc #>10
		bcc sds
		txa
		cpy #<100
		sbc #>100
		bcc sds2
		txa
		cpy #<1000
		sbc #>1000
		bcc sds3
		txa
		cpy #<10000
		sbc #>10000
		bcc sds5
		lda #$14
		gcs sds4

sds		lda #$20
		pha			;x
sds2		lda #$20
		pha			;x
sds3		lda #$20
sds4		pha			;x

sds5		lda #$22
		pha

		ldx #240
ncvbx		lda direntry_name_exp-240,x
		beq ncvbx2
		pha			;x
		inx
		bne ncvbx
		lda #$22
		pha
		gne bde
ncvbx2		lda #$22
		pha
		lda #$20
ncvbx4		pha			;x
		inx
		bmi ncvbx4

bde		.if SUPERCPU=1
		ldx #$2a		;*
		lda direntry_attrib_exp	;closed?
		tay
		bpl *+4
		ldx #$20
		phx			;x
                .else
                lda #$2a		;*
                ldy direntry_attrib_exp
                bpl *+4
		lda #$20
		pha			;x
                .fi

		ldx #253
cbe		lda direntry_filetype_exp-253,x
		beq dirdata
		pha			;x
		inx
		bne cbe
dirdata		tya
		and #%01010000
		cmp #%01010000		;Deleteable & writeable?
		lda #"<"
		bcc *+4
		lda #$20
		pha

jrr		lda #$20
jr		pha			;x
		.if SUPERCPU=1
		tsc
		eor direntry_spc_exp
		bne jrr
		.else
		tsx
		cpx direntry_spc_exp
		bne jr
		lda #0
		.fi
		pha			;x EOL
		ldy #31
		ldx #32
		jmp start_zeropage

flushdir        #dec_l_a direntry_number_exp
		bmi xx
		jsr OP_read_to_direntry
		#jmps flushdir
xx		rts

dirlabel_text	.byte $00,$34,$36,$45,$44,$49,$20,$22;" IDE64
blkused_text	.byte $20,".","D","E","S","U",$20,"S","K","C","O","L","B"
prtused_text	.byte $20,$20,".","S","N","O","I","T","I","T","R","A","P"
		.pend

;---------------------------------------
;Read 32 bytes to direntry_exp C=1 if error
;---------------------------------------
OP_read_to_direntry .proc
		.if SUPERCPU=1
		lda direntry_mode_exp
                .else
		ldx direntry_mode_exp
                .fi
		bne ofs
;-----------------
;CFS filesystem is the simplest
;
		ldx #224
cb2		lda data_g		;DATA PORT
		lda low_reg		;0-7 BIT
		sta direntry_exp-224,x
		inx
		lda hig_reg		;8-15 BIT
		sta direntry_exp-224,x
		inx
		bne cb2
                lda direntry_number_exp
                lsr
                lsr
                tax
		lda direntry_nexts_exp
		asl
		asl
		asl
		rol next_sector,x
		asl
		rol next_sector,x
		rts
					;Hack to read old filesystem
;-----------------
;v0.02 filesystem
;
ofs		.if SUPERCPU=1
		dec a
                .else
		dex
                .fi
		bne partition
		lda direntry_number_exp
		bne kqj
		sta direntry_attrib_exp

		ldx #12			;Skip until end of sector
skip		bit data_g
		dex
		bne skip
		rts

o_sys		= 11

t1		.text "DSPURCDU",0,"CNS"     ;SYS, NAT, 41, 71, 81, C81, PRN, FOR
t2		.text "EERSEBIN",0,"FAY"
t3		.text "LQGRLMRK",0,"STS"
t4		.byte FTYPE_DEL,FTYPE_NORMAL,FATTR_EXEC | FTYPE_NORMAL,FTYPE_NORMAL
		.byte FTYPE_NORMAL,FTYPE_NORMAL,FATTR_EXEC | FTYPE_DIR,FTYPE_NORMAL

kqj		eor #15
		bne nueu		;Not first

		ldx #2
		bit data_g		;($00)
		bit low_reg		;last sector?
		bvc uxjkx
		sta next_sector
		sta next_sector+1
		sta next_sector+2
		inx
		gne uuee

uxjkx		lda low_reg
		and #%00111111
		sta next_sector+1
		lda hig_reg
		sta next_sector+2
		lda data_g		;($02)
		lda low_reg
		and #15
		sta next_sector
		lda hig_reg
uuee		sta next_sector+3

		jsr skip		;($06)

nueu		lda data_g		;Attrib ($00)
		#ldatax low_reg
		and #3
		tay
		txa
		and #%00010000
		beq *+4			;DIR?
		ldy #6
		jsr fillftype
		txa
		pha
		and #%11000000
		eor #%01100000
		asl
		bpl *+4			;Write protect?
		ora #%00100000
		ror
		ora t4,y
		cpy #6			;DIR?
		bne ded3
		tax
		bmi ded3
		#lda0			;*DIR????
ded3
		sta direntry_attrib_exp

		#lda0			;TODO: convert date
		sta direntry_date_exp
		sta direntry_date_exp+1
		sta direntry_date_exp+2
		sta direntry_date_exp+3

		jsr getname

		pla
		cmp #$f0
		bne tre2
		lda direntry_attrib_exp	;Dirlabel needs padding...
		and #%01111111
		sta direntry_attrib_exp
		gcs tre
					;Kill $a0 padding of name
tre2		ldx #15
ktt		lda direntry_name_exp,x
		eor #$a0
		bne tre
		sta direntry_name_exp,x
		dex
		bpl ktt
tre
		ldx #direntry_size_exp-direntry_exp
		jsr getword		;($12)
		jsr getnextword		;($14)
		ldx #3
		jsr skip
		lda data_g		;($1c)
		lda low_reg
		sta direntry_pointer_exp+1
		lda hig_reg
		sta direntry_pointer_exp+2
		lda data_g		;($1e)
		lda low_reg
		and #15
		sta direntry_pointer_exp
		lda hig_reg
		sta direntry_pointer_exp+3
		rts

fillftype	lda t1,y
		sta direntry_filetype_exp
		lda t2,y
		sta direntry_filetype_exp+1
		lda t3,y
		sta direntry_filetype_exp+2
		rts

getnextword	inx
		inx
getword		lda data_g		;($12)
		lda hig_reg
		sta direntry_exp+1,x
		lda low_reg
		sta direntry_exp,x
		rts

getname		ldx #240
cb23		lda data_g		;Get name ($02)
		lda low_reg
		sta direntry_name_exp-240,x
		inx
		lda hig_reg
		sta direntry_name_exp-240,x
		inx
		bne cb23
		rts
;-----------------
;Partition table
;
partition	.if SUPERCPU=1
		dec a
                .else
		dex
                .fi
		bne partition2
                .if SUPERCPU=1
		sta direntry_attrib_exp
                .else
		stx direntry_attrib_exp
                .fi
		lda direntry_number_exp
		bne be
		lda #DIRMODE_PARTITION_END
		sta direntry_mode_exp
en2     	rts
be		eor #15
		bne en
		bit data_g
		bit data_g
		lda hig_reg
		sta direntry_defpart_exp
		ldx #10
		jsr skip
		ldx #0
		jsr getnextsector
		ldx #2
		jsr getnextsector
		jsr skip
		jsr getname
		lda #FTYPE_DIR|FATTR_READ
		sta direntry_attrib_exp
		ldx #232
		gne skip
                
en		eor #1
		bne en2
                lda setup_atribut2_exp
                bpl en2
                ldx #3
lk              #lda0
		sta direntry_size_exp,x
                dex
                bpl lk
                sta direntry_pointer_exp
                lda #FATTR_CLOSED | FTYPE_NORMAL
                sta direntry_attrib_exp
                ldy #o_sys
                jsr fillftype
                ldx #6
lk2             lda koo2,x
                sta direntry_name_exp,x
                dex
                bpl lk2
                rts
koo2		.text "SYSTEM",0

getnextsector	lda data_g
		lda low_reg
		sta next_sector,x
		lda hig_reg
		sta next_sector+1,x
		rts

partition2     	.if SUPERCPU=1
		dec a
                .else
		dex
                .fi
		bne isodir		;isodir
                .if SUPERCPU=0
		lda #0
                .fi
		ldx #3
vkf		sta next_sector,x
		sta direntry_size_exp,x
		dex
		bpl vkf
		sta direntry_pointer_exp
		sta direntry_filetype_exp
		lda #16
		sec
		sbc direntry_number_exp
		.if DIRECTORY_BLOCKSIZE_256=0
		asl
		.fi
		sta direntry_size_exp+1
		jsr getname
		lda direntry_number_exp
		eor #15
		eor direntry_defpart_exp
		cmp #1
		bit data_g
		lda low_reg
		bpl nn			;not valid
		and #%00110000
		beq nwrite
		and #%00100000
		bne nn			;hidden
		lda #(FTYPE_NORMAL|FATTR_DELETE|FATTR_WRITE|FATTR_READ|FATTR_EXEC) << 1
		.byte SKIP2
nwrite		lda #(FTYPE_NORMAL|FATTR_DELETE|FATTR_READ|FATTR_EXEC) << 1
		ror
                #inc_l_ direntry_partnum
		.byte SKIP2
nn		lda #0
		sta direntry_attrib_exp
		bit data_g
		bit data_g
		lda low_reg
		ldy #8
		and #%10110000
		beq bd			;unformatted
		cmp #%00010000
		beq bd2			;cfs
		dey			;unknown
		gne bd
bd2		iny
		lda setup_atribut2_exp
                bpl bd
                iny			;nat
bd		jsr fillftype
		ldx #5
		gne skip
;-----------------
;ISO directory
;
isodir		
		lda direntry_number_exp
                eor #15
                bne ny
                sta direntry_iso_bytes_exp
                sta direntry_iso_bytes_exp+1

ny		lda data_g
                lda low_reg
                bne ny2
                
                clc
		sbc direntry_iso_bytes_exp
                tax
                beq nxp
                jsr skip
nxp             lda #>1024
                sbc direntry_iso_bytes_exp+1
                tay
                beq nxp2
nxp3            jsr skip
                dey
                bne nxp3
nxp2		tya
		sta direntry_number_exp
		sta direntry_attrib_exp
                clc
                #dec_l_a direntry_iso_dirsectors_exp
                bne ry	;A=0
                sec
ry		ldx #3
		.if SUPERCPU=1
                tdc
                .fi
ly		bcs tu
		inc next_sector,x
		bne lyy
tu              sta next_sector,x
                dex
                bpl ly
lyy		rts
                
ny2             lsr
                tax
                adc direntry_iso_bytes_exp
                sta direntry_iso_bytes_exp
                .if SUPERCPU=1
                lda direntry_iso_bytes_exp+1
                adc #0
                sta direntry_iso_bytes_exp+1
                .else
                bcc nx
                inc direntry_iso_bytes_exp+1
nx
		.fi
		txa
                cmp #17
                bcc ezmi
                pha
                lda data_g
                lda data_g
                ldx #direntry_pointer_exp-direntry_exp
                jsr getword
                ora #$40
                sta direntry_pointer_exp
                jsr getnextword
                ldx #direntry_date_exp-direntry_exp
                jsr getword
                jsr getnextword
                ldx #5
                jsr skip
                lda data_g
                lda hig_reg
                sta direntry_attrib_exp
                ldx #3
                jsr skip
                ldx #236
                lda data_g
                ldy low_reg
                cpy #1
                bne np
                lda hig_reg
                sta direntry_name_exp
                gcs ent
np              lda direntry_mode_exp
                cmp #DIRMODE_ISO
                beq bj

bj2u            lda data_g
		pla
                #sbc1c
                pha
                lda low_reg
                bmi ala
                cmp #32
                bcc ala
                cmp #":"
                beq ala
                cmp #"="
                beq ala
                cmp #65
                bcc jch
                cmp #91
                bcc nagy
                cmp #95
                beq ala
                cmp #97
                bcc jch
                cmp #123
                bcs jch
                sbc #32-1
                .byte SKIP2
ala		lda #164
		.byte SKIP2
nagy            ora #$80
jch             sta direntry_name_exp-236,x
                inx
                beq udyy
teb             dey
                dey
                bne bj2u
                tya
		sta direntry_name_exp-236,x
                geq txe

udyy		cpy #2
		beq txe
		ldx #253
spx		lda direntry_name_exp-236,x
                cmp #","
                beq tbj
                cmp #"."
                bne ipx
tbj             #phy_a
		ldy #252
thi		lda direntry_name_exp-236,x
                #sta_l_y_ direntry_name_exp-236
                iny
                inx
                bne thi
                .if SUPERCPU=1
                tyx
                ply
                .else
                tya
                tax
                pla
                tay
                .fi
                gcs teb
                
ipx		inx
		bne spx

udyye		dey
		dey
                beq txe
                lda data_g
		pla
                #sbc1c
                pha
                lda low_reg
                cmp #","
                beq tbj2
                cmp #"."
                bne udyye
tbj2            ldx #252
                gne jch
                
txe
		ldx #236
rekq            lda direntry_name_exp-236,x
		beq ene
                cmp #";"
                bne xoq
                lda direntry_name_exp-235,x
		eor #"1"
                bne yfq
                cpx #254
                beq enei
                lda direntry_name_exp-234,x
                bne yfq
enei            sta direntry_name_exp-236,x
                geq ene
                
xoq             cmp #$5f	;_
		beq huzq
                cmp #","
                beq ftp
		cmp #"/"
                bne xfq
yfq             lda #"."
                .byte SKIP2
huzq            lda #$a4
                sta direntry_name_exp-236,x
xfq             inx
                bne rekq

ene             dex
		cpx #236
                bcc cyp
		lda direntry_name_exp-236,x
                eor #"."
                beq enei
                inx
ukek            dex
		cpx #237
                bcc cyp
		lda direntry_name_exp-236,x
                cmp #32
                beq cyp
                eor #"."
                bne ukek
                geq ftp
                
cyp		ldx #255
                gne xt
;-----------------
bj2             lda data_g
		pla
                #sbc1c
                pha
                lda low_reg
                sta direntry_name_exp-236,x
                inx
                beq udy
                dey
                beq udy2
bj              lda hig_reg
                sta direntry_name_exp-236,x
                inx
                beq udy
                dey
                bne bj2
udy2		tya
		sta direntry_name_exp-236,x
udy
		ldx #236
rek             lda direntry_name_exp-236,x
		beq eni
                cmp #";"
                bne xo
                lda direntry_name_exp-235,x
		cmp #"1"
                bne yf
                lda direntry_name_exp-234,x
                bne yf
                sta direntry_name_exp-236,x
                geq eni
                
xo              cmp #$5f	;_
		beq huz
                cmp #"."
                beq ftp
		cmp #"/"
                bne xf
yf              lda #"."
                .byte SKIP2
huz             lda #$a4
                sta direntry_name_exp-236,x
xf              inx
                bne rek
eni             ldx #255
                gne xt

ftp		#lda0
		sta direntry_name_exp-236,x
xt		ldy #0
ftpl		inx
                beq ru
		lda direntry_name_exp-236,x
                cmp #","
                beq nd
                cmp #"<"
                beq nd
                cmp #" "
                beq nd
                cmp #";"
                bne tkq
nd              ldx #0
ru              txa
                dex
tkq             #sta_l_y_ direntry_filetype_exp
                iny
                cpy #3
                bcc ftpl

ent
		pla
                sec
                sbc #17
ezmi            tax
                beq xj
                jsr skip
xj
		.if SUPERCPU=1
		lda direntry_name_exp
                .else
                ldx direntry_name_exp
                .fi
                bne noilabel
                lda direntry_date_exp+1
                lsr
                lsr
                lsr
                sta direntry_iso_dirsectors_exp
                ldx #16
                lda #32
li3             sta direntry_name_exp,x
                dex
                bpl li3
                lda #FTYPE_DIR|FATTR_READ|FATTR_EXEC
                gne nulldirin
noilabel
		.if SUPERCPU=1
                dec a
                .else
                dex
                .fi
                bne noilabel2
                lda #"."
                sta direntry_name_exp
                sta direntry_name_exp+1
                #lda0
                sta direntry_name_exp+2

nulldir         ldx #3
                #lda0
xq              sta direntry_size_exp,x
                dex
                bpl xq
                lda #FTYPE_DIR|FATTR_READ|FATTR_EXEC|FATTR_CLOSED
nulldirin       jsr dl
		ldy #6
		jmp fillftype

noilabel2	lda direntry_attrib_exp
                and #%00000010
                bne nulldir
                ldx #3
ro              lda direntry_date_exp,x
                sta direntry_size_exp,x
                dex
                bpl ro
                lda #FTYPE_NORMAL|FATTR_READ|FATTR_EXEC|FATTR_CLOSED
dl              sta direntry_attrib_exp
		lda #8
                sta direntry_number_exp
		rts
		.pend
